2####################################################
#########N#dhhyyyhdN############Ndhyyyhhd#N#########
######ysooooooooooooy#########dsoooooooooooosh######
####soooooooooooooooooosyyyysoooooooooooooooooos####
##doooooooooooooooooooooooooooooooooooooooooooooo###
##sooooyd##NN###dooooooooooooooooood###NN##dsoooos##
##oooos##########soooooossssooooooy##########oooos##
##ooooo##########sooooos####ooooooy##########oooos##
##yoooo##########ooooooy####sooooos#########Nooooy##
##hoooo#########Nooooooh####hoooooo#########dooood##
###ooooosyyhhdddsooooooN#####ooooooydddhhyysoooooN##
###hooooooooooooooooooy######sooooooooooooooooood###
#####yoooooooooooooooy#######NsoooooooooooooooyN####
########dhhyyssssssh############yssssssyyhhd########
####################################################
##                                                ##
##                 Script: CopyAppStacksv4.ps1    ##
##                 Author: Laurens van Duijn      ##
##                Version: 1.0                    ##
##           Release Date: February 2020          ##
##                                                ##
####################################################
####################################################
####################################################

# This script will allow you to copy any appstack, even from a VSAN datastore to any other available (shared) datastore under the same vCenter.

#Guide
#1. Change vCenter Address to match yours
#2. Add vCenter DataCenter name
#3. Change paths to your datastores
#4. Set AppVolumes version by uncommenting the version you want to use

#Changelog
#v1 added metadata option
#v2 merged metadata copy with appstack copy
#v3 added variables
#v4 added App Volumes 4.0 support. Script is by default App Volumes 4.x

####################################################

#Uncomment below  for App Volumes 2.x
#$AppVolumes = "cloudvolumes/apps"

#Uncomment below for App Volumes 4.x
$AppVolumes = "appvolumes/packages"

$vCenterFQDN = "Enter vCenter Address"
$DataCenter = "Enter datacenter name"
$DataStoreTest = "Enter source Datastore"
$DataStoreProd = "Enter destination Datastore"

####################################################

function Show-Menu
{
     param (
           [string]$Title = 'Copy AppStacks to Production'
     )
     cls
     Write-Host "================ $Title ================"
     
     Write-Host "Press '1' To Connect to vCenter."
     Write-Host "Press '2' To List AppStacks in Test."
	 Write-Host "Press '3' To List AppStacks Production."
     Write-Host "Press '4' To Copy AppStacks to Production."
     Write-Host "Press '5' To Backup AppStacks."	 	 
     Write-Host "Press '6' To Get Support."	 
     Write-Host "Press 'Q' To Quit."
}
do
{
     Show-Menu
     $input = Read-Host "Please make a selection"
     switch ($input)
     {
           '1' { Connect-VIServer -Server $vCenterFQDN
           } '2' { Get-HardDisk -Datastore $DataStoreTest -DatastorePath "[$DataStoreTest] $AppVolumes" | Format-Table -AutoSize -Property Filename,CapacityGB
		   } '3' { Get-HardDisk -Datastore $DataStoreProd -DatastorePath "[$DataStoreProd] $AppVolumes" | Format-Table -AutoSize -Property Filename,CapacityGB
           } '4' { $AppStacks = Get-HardDisk -Datastore $DataStoreTest -DatastorePath "[$DataStoreTest] $AppVolumes"
$menu = @{}
for ($i=1;$i -le $AppStacks.count; $i++) {
    Write-Host "$i. $($AppStacks[$i-1].name.Split('.')[0])"
	    $menu.Add($i,($AppStacks[$i-1].name.Split('.')[0]))
		    }
			
			[int]$ans = Read-Host 'Enter selection'
			$selection = $menu.Item($ans)
			Get-HardDisk -Datastore "$DataStoreTest" -DatastorePath "[$DataStoreTest] $AppVolumes/$selection.vmdk" | Copy-HardDisk -Force "[$DataStoreProd] $AppVolumes" | Format-Table -AutoSize -Property Filename,CapacityGB
			Copy-DatastoreItem -force vmstores:\$vCenterFQDN@443\$DataCenter\$DataStoreTest\$AppVolumes\$selection.vmdk.metadata vmstores:\$vCenterFQDN@443\$DataCenter\$DataStoreProd\$AppVolumes\$selection.vmdk.metadata
            Copy-DatastoreItem -force vmstores:\$vCenterFQDN@443\$DataCenter\$DataStoreTest\$AppVolumes\$selection.json vmstores:\$vCenterFQDN@443\$DataCenter\$DataStoreProd\$AppVolumes\$selection.json
		   } '5' { Get-HardDisk -Datastore "$DataStoreTest" -DatastorePath "[$DataStoreTest] $AppVolumes/*" | Copy-HardDisk -Force "[$DataStoreProd] Backup/"
		   } '6' { Write-Host "#NoSupport! Build by @LaurensvanDuijn @ITQ. Updates come when bugs are reported."
           } 'q' {
                return
           }
     }
     pause
}
until ($input -eq 'q')
